searchData={"items":[{"type":"protocol","title":"LiveUI","doc":"Protocol for `Ecto.Schema` struct to configure related `Phoenix.LiveView` modules.","ref":"LiveUI.html"},{"type":"protocol","title":"Simple setup - LiveUI","doc":"defimpl LiveUI, for: MyApp.Admin.Company do\n      use LiveUI.Protocol\n    end\n\nCalling `use LiveUI.Protocol` will define all function which are implementing the `LiveUI` protocol.\nIt is a convenience to eliminate the boilerplate code required by the protocol. Most of the values set\nby the macro are the ones you would expect, like using the first string field as a title or building\nrelation links using foreign keys. When that is not the case, we can override these functions to change\nhow the data will be presented and processed.","ref":"LiveUI.html#module-simple-setup"},{"type":"protocol","title":"Custom setup - LiveUI","doc":"defimpl LiveUI, for: MyApp.Admin.User do\n      use LiveUI.Protocol,\n        flop: [\n          filterable: [:name, :email, :activated_at],\n          sortable: [:activated_at],\n          default_order: %{order_by: [:activated_at], order_directions: [:desc]}\n        ]\n\n      def title(user), do: record.name\n      def description(user), do: user.email\n      def filter_operators(_), do: [age: [:<=, :>=]]\n      def input_hints(_), do: [website: \"Should begin with https://\"]\n      def uploads(_), do: [image: [accept: ~w(.jpg .jpeg),max_file_size: 800_000]]\n\n      def index_view(user) do\n        super(user)\n        |> ignore_fields(:new, [:confirmed_at])\n        |> add_batch_action(:deactivate, \"Deactivate\", MyAppWeb.Admin.UserLive.Deactivate)\n        |> add_formatters(website: {&link_/1, %{name: \"Web\"}})\n      end\n\n      def show_view(user) do\n        super(user)\n        |> add_formatters(bio: &markdown/1, website: &link_/1)\n      end\n    end\n\nNOTE: `flop` option will derive `Flop.Schema` protocol unless it is already derived in Ecto schema module:\n\n    Protocol.derive(Flop.Schema, MyApp.Admin.User,\n      filterable:  ,\n      sortable: [<:updated_at or :id>],\n      default_order: %{order_by: [<:updated_at or :id>], order_directions: [:desc]}\n    )\n\n`:updated_at` is the only sort filed set with a fallback to `:id` if it doesn't exist.\nAlso, all fields are searchable by default. Please check `Flop.Schema` for more info.","ref":"LiveUI.html#module-custom-setup"},{"type":"protocol","title":"Extending live view modules - LiveUI","doc":"Customizing default views via protocol is sometimes not enough as we might need extra markup and logic in these views.\n\nDefault template can be extended by overriding `c:Phoenix.LiveView.render/1` callback. We could do the same\nwith `c:Phoenix.LiveView.mount/3` if we need to put extra assigns into socket. Finally, the data\ncould be processed with custom `c:Phoenix.LiveView.handle_event/3` callbacks.\n\n    defmodule LiveUIWeb.Admin.UserLive.Index do\n      use LiveUI.Views.Index, for: LiveUI.Admin.User\n\n      def mount(params, session, socket) do\n        {:ok, socket} = super(params, session, socket)\n        {:ok, socket |> assign(:greeting, \"Hello\")}\n      end\n\n      def render(assigns) do\n        ~H\"\"\"\n        <.p phx-click=\"bigger-greeting\"><%= @greeting %> from extra markup before the table!</.p>\n        <%= super(assigns) %>\n        \"\"\"\n      end\n\n      def handle_event(\"bigger-greeting\", _, socket) do\n        {:noreply, socket |> assign(greeting: String.upcase(socket.assigns.greeting))}\n      end\n    end","ref":"LiveUI.html#module-extending-live-view-modules"},{"type":"function","title":"LiveUI.description/1","doc":"Defines a description for the record, it is used in HEEX templates.\n\nDefault implementation is the value of the second string field from the Ecto struct.","ref":"LiveUI.html#description/1"},{"type":"function","title":"LiveUI.filter_operators/1","doc":"Configures `t:Flop.Filter.op/0` operators for fields in search form.\n\nDefault operator for string fields is `:ilike`, for numbers and dates it is `:==`.","ref":"LiveUI.html#filter_operators/1"},{"type":"function","title":"Example - LiveUI.filter_operators/1","doc":"# add extra inputs to search form\n    def filter_operators(_user), do: [age: [:<=, :>=]]\n\nNOTE: This setting is not applicable to boolean and enum fields; they are always rendered as drop-down list.\nAlso, any searchable foreign key field is rendered as `LiveSelect` component.","ref":"LiveUI.html#filter_operators/1-example"},{"type":"function","title":"LiveUI.heading/1","doc":"Renders `Show` page heading as a component instead of `title/1`.\n\nDefault implementation is `false` which will prevent it from rendering.","ref":"LiveUI.html#heading/1"},{"type":"function","title":"Example - LiveUI.heading/1","doc":"use Phoenix.Component\n    import LiveUI.Components.Core\n\n    def heading(assigns) do\n      ~H\"\"\"\n      <.h3><%= @name %></.h3>\n      <.p><%= @email %></.p>\n      \"\"\"\n    end\n\nNOTE: `p` and `h3` components are from `LiveUI.Components.Core` which delegates the calls to `PetalComponents`.","ref":"LiveUI.html#heading/1-example"},{"type":"function","title":"LiveUI.ignored_fields/1","doc":"List of ignored fields, it is applied to both `Index` and `Show` views and their forms.\n\nNOTE: fields can be also ignored for all schema modules via application config:\n\n      config :live_ui,\n        ignored_fields: [:token, :first_version_id, :current_version_id]","ref":"LiveUI.html#ignored_fields/1"},{"type":"function","title":"LiveUI.index_view/1","doc":"Configuration that controls how the data is displayed in the table and how is processed with create form.\nIt also holds an info about custom actions, ie. `Deactivate` action in the example.\n\n    # LiveUI.index_view(%User{})\n    [\n      formatters: [website: {&LiveUI.Formatters.link_/1, %{name: \"Web\"}}],\n      actions: [\n        new: [\n          name: \"New user\",\n          allowed: true,\n          fields: [:name, :email, :bio, :age, :website, :company_id, :department_id, :role, :active],\n          optional_fields: [:age],\n          inputs: [],\n          function: &LiveUI.Queries.create/2,\n          changeset: &LiveUI.Changeset.create_changeset/3,\n          validate_changeset: &LiveUI.Changeset.create_changeset/3\n        ]\n      ],\n      batch_actions: [\n        deactivate: [\n          name: \"Deactivate\",\n          component: LiveUIWeb.Admin.UserLive.Deactivate\n        ],\n        delete: [\n          name: \"Delete\",\n          allowed: true,\n          function: &LiveUI.Queries.delete_ids/2\n        ]\n      ],\n      fields: [:id, :name, :email, :website, :company_id, :department_id, :role, :active, :confirmed_at],\n      preload: [:company, :department],\n      function: &LiveUI.Queries.find_by_filter/3\n    ]\n\nThis structure can be updated directly or by using built-in functions from `LiveUI.Protocol.Utils`.\n\nUse `put_in/3` function:\n\n    # disable update form\n    def show_view(session) do\n      super(session)\n      |> put_in([:actions, :edit, :allowed], false)\n    end\n\n    # override changeset function for create form\n    def index_view(contact) do\n      super(contact)\n      |> put_in([:actions, :new, :changeset], &MyApp.Member.Contact.create_changeset/3)\n    end\n\n    # create_changeset in my_app/member/contact.ex\n    def create_changeset(contact, params, socket) do\n      LiveUI.Changeset.create_changeset(contact, params, socket)\n      |> validate_format(:email, ~r/^[^ ]+@[^ ]+$/, message: \"must have the @ sign and no spaces\")\n    end\n\nUse `LiveUI.Protocol.Utils.ignore_fields/2` helper function:\n\n    def index_view(user) do\n      super(user)\n      |> ignore_fields([:updated_at, :inserted_at])\n    end","ref":"LiveUI.html#index_view/1"},{"type":"function","title":"Formatters - LiveUI.index_view/1","doc":"List of formatters to control how the fields are displayed on the screen.\nThey are added with `LiveUI.Protocol.Utils.add_formatters/2`.","ref":"LiveUI.html#index_view/1-formatters"},{"type":"function","title":"Actions - LiveUI.index_view/1","doc":"Built-in and custom actions are rendered inside the modal.\nCustom actions are added with `LiveUI.Protocol.Utils.add_action/5`.\n\n#","ref":"LiveUI.html#index_view/1-actions"},{"type":"function","title":"New action - LiveUI.index_view/1","doc":"Built-in action that renders a form to create records.\n\n- name\n\n  Used as text in action button and page title.\n\n- allowed\n\n  Boolean or a function to allow access to the action.\n  The function accepts socket assign and should return a boolean.\n\n- fields\n\n  List of fields which are rendered in the form.\n  All fields are included unless ignored with `LiveUI.Protocol.Utils.ignore_fields/3`.\n\n- optional_fields\n\n  All fields are required unless they are set as optional with `LiveUI.Protocol.Utils.set_optional_fields/3`.\n\n- inputs\n\n  Configures form input types via `LiveUI.Protocol.Utils.configure_inputs/3`.\n\n- function\n\n  Function that saves the record. Override if extra logic is needed.\n\n- changeset\n\n  Changeset function for phx-submit that validates all fields as required.\n  Override to add custom validation.\n\n- validate_changeset\n\n  Changeset function for phx-change that points to changset for phx-submit by default.\n  Override if it behaves differently.","ref":"LiveUI.html#index_view/1-new-action"},{"type":"function","title":"Batch actions - LiveUI.index_view/1","doc":"Built-in and custom batch actions are rendered inside the modal and they operate on selected records.\nCustom batch actions are added with `LiveUI.Protocol.Utils.add_batch_action/5`.\n\n#","ref":"LiveUI.html#index_view/1-batch-actions"},{"type":"function","title":"Delete action - LiveUI.index_view/1","doc":"Built-in batch action that renders a dialog to delete selected records.\n\n- name\n\n  Used as text in action button and page title.\n\n- allowed\n\n  Boolean or a function to allow access to the action.\n  The function accepts socket assign and should return a boolean.\n\n- function\n\n  Function that deletes selected records. Override if extra logic is needed.","ref":"LiveUI.html#index_view/1-delete-action"},{"type":"function","title":"Fields - LiveUI.index_view/1","doc":"List of fields to display in the table. Defaults to all fields.\nFields could be removed from the list with `LiveUI.Protocol.Utils.ignore_fields/2`.","ref":"LiveUI.html#index_view/1-fields"},{"type":"function","title":"Preload - LiveUI.index_view/1","doc":"List of parent relation that are preloaded and shown in the table with links to their own `Show` view.\nDefaults to all `belongs_to` relations.","ref":"LiveUI.html#index_view/1-preload"},{"type":"function","title":"Function - LiveUI.index_view/1","doc":"Function that loads the records which are filtered with `Flop.Filter` via search form.\nIt will scope the result to current user if configured with `ownership/1`","ref":"LiveUI.html#index_view/1-function"},{"type":"function","title":"LiveUI.input_hints/1","doc":"Add input_hints to form inputs.","ref":"LiveUI.html#input_hints/1"},{"type":"function","title":"Example - LiveUI.input_hints/1","doc":"def input_hints(_user), do: [website: \"Should begin with https://\"]","ref":"LiveUI.html#input_hints/1-example"},{"type":"function","title":"LiveUI.namespace/1","doc":"Name of the parent namespace for nested routes.\n\nDefault is the name of second to last module of the Ecto struct.\nFor example, `MyApp.Admin.User` will produce routes starting with `/admin/users`.","ref":"LiveUI.html#namespace/1"},{"type":"function","title":"LiveUI.ownership/1","doc":"Configures ownership relation for database queries, default is false.\n\nThis will scope queries to `:user_id` foreign key that is equal to `:id` key of the `:current_user` socket assign:\n\n    def ownership(_contact), do: {:user_id, :current_user}","ref":"LiveUI.html#ownership/1"},{"type":"function","title":"LiveUI.resource/1","doc":"Resource name that is used in routes and templates.\n\nDefault is the name of the last module of the Ecto struct.","ref":"LiveUI.html#resource/1"},{"type":"function","title":"LiveUI.resources/1","doc":"Resource name in plural that is used in routes and templates.\n\nDefault is the name of the last module of the Ecto struct with appended `s` character.\nThis should be overriden for plurals like `companies` or `fish`.","ref":"LiveUI.html#resources/1"},{"type":"function","title":"LiveUI.search_field_for_description/1","doc":"The field name selected in query which value is shown in `LiveSelect` dropdown list as record description.\n\nDefault is the description field.","ref":"LiveUI.html#search_field_for_description/1"},{"type":"function","title":"LiveUI.search_field_for_title/1","doc":"The field name selected in query which value is shown in `LiveSelect` dropdown list as record title.\n\nDefault is the title field.","ref":"LiveUI.html#search_field_for_title/1"},{"type":"function","title":"LiveUI.search_function/1","doc":"Search function used by `LiveSelect` to search for parent records in forms.\n\nIt uses configured `search_field_for_title/1` and `search_field_for_description/1` fields in select statement.\n\nDefault implementation will scope the result to current user from the socket assign if configured via `ownership/1`.\nIt will also apply the scope of selected parent to other relations if they share the same parent as the record\nbeing created, ie. when selecting a company with one LiveSelect input only departments from that company will be\nsearched in second LiveSelect input.\n\nCustom function accepts socket assigns and search term and should return a map with id, label and description\nfields used in `LiveSelect` dropdown list.\n\n    def search_function(_department), do: &search_by_name/2\n\n    # scope department search to current user's company\n    def search_by_name(assigns, text) do\n      import Ecto.Query\n      company_id = Map.get(assigns[:current_user], :company_id)\n      ilike = \"#{text}%\"\n\n      from(d in MyApp.Department,\n        where: [company_id: ^company_id],\n        where: ilike(d.name, ^ilike),\n        select: %{\n          value: d.id,\n          label: d.name,\n          description: d.location\n        },\n        limit: 5\n      )\n      |> LiveUI.Repo.all()\n    end\n\nDefault implementation will apply the same scope to `:company_id` since `User` and `Department`\nschemas have the same `belongs_to :company` relation.","ref":"LiveUI.html#search_function/1"},{"type":"function","title":"LiveUI.show_view/1","doc":"Configuration that controls how the data is displayed for a single record and how is processed with edit form.\nThe fields are documented in `LiveUI.index_view/1`.\n\n    # LiveUI.show_view(%User{})\n    [\n      formatters: [\n        email: &LiveUI.Formatters.copy/1,\n        bio: &LiveUI.Formatters.markdown/1,\n        website: &LiveUI.Formatters.link_/1,\n        role: &String.upcase/1\n      ],\n      actions: [\n        edit: [\n          name: \"Edit\",\n          allowed: true,\n          fields: [:name, :email, :bio, :age, :website, :company_id, :department_id, :role, :active, :confirmed_at],\n          optional_fields: [],\n          inputs: [],\n          function: &LiveUI.Queries.update/2,\n          changeset: &LiveUI.Changeset.update_changeset/3,\n          validate_changeset: &LiveUI.Changeset.update_changeset/3\n        ],\n        delete: [\n          name: \"Delete\",\n          allowed: true,\n          function: &LiveUI.Queries.delete/2\n        ]\n      ],\n      fields: [:id, :name, :email, :bio, :age, :website, :company_id, :department_id, :role, :active, :confirmed_at, :inserted_at, :updated_at],\n      preload: [:company, :department],\n      function: &LiveUI.Queries.find_by_id/2\n    ]","ref":"LiveUI.html#show_view/1"},{"type":"function","title":"LiveUI.title/1","doc":"Defines a title for the record, it is set as page title and used in HEEX templates.\n\nDefault is the value of the first string field from the Ecto struct.\n\n    # custom field\n    def title(user), do: record.name\n\n    # computed value\n    def title(user), do: \"#{user.first_name} #{user.last_name}\"","ref":"LiveUI.html#title/1"},{"type":"function","title":"LiveUI.uploads/1","doc":"Configures upload fields. For single file upload use `Ecto` string type; for multiple files use array of strings.","ref":"LiveUI.html#uploads/1"},{"type":"function","title":"Examples - LiveUI.uploads/1","doc":"def uploads(_) do\n      [\n        # ecto string\n        image: [\n          accept: ~w(.jpg .jpeg),\n          max_file_size: 800_000\n        ],\n        # ecto array of strings\n        extra_images: [\n          accept: ~w(.jpg .jpeg),\n          max_entries: 3,\n          max_file_size: 800_000\n        ]\n      ]\n    end\n\nIn case of local upload, files are saved using `priv/static/uploads/ / / -  path`,\npath for `image` field in `product` schema might look like this:\n\n    priv/static/uploads/product/image/1a64228b-54a4-4824-a90f-b236806aecbb-elixir.jpg\n\nwhich is served from this url:\n\n    http://localhost:4010/uploads/product/image/1a64228b-54a4-4824-a90f-b236806aecbb-elixir.jpg","ref":"LiveUI.html#uploads/1-examples"},{"type":"function","title":"Configure static path and live reloader - LiveUI.uploads/1","doc":"To enable serving from `uploads` folder add it to the `static_paths` list in `my_app_web.ex`:\n\n    def static_paths, do: ~w(assets fonts images favicon.ico robots.txt uploads)\n\nIn `dev.ex` disable live reload when files are added to `uploads` folder:\n\n    ~r\"priv/static/(?!uploads).*(js|css|png|jpeg|jpg|gif|svg)$\"","ref":"LiveUI.html#uploads/1-configure-static-path-and-live-reloader"},{"type":"type","title":"LiveUI.t/0","doc":"All the types that implement this protocol.","ref":"LiveUI.html#t:t/0"},{"type":"module","title":"LiveUI.Components.Core","doc":"Core components.","ref":"LiveUI.Components.Core.html"},{"type":"function","title":"LiveUI.Components.Core.action_button/1","doc":"Renders action button.","ref":"LiveUI.Components.Core.html#action_button/1"},{"type":"function","title":"Attributes - LiveUI.Components.Core.action_button/1","doc":"* `config` (`:list`)\n* `action` (`:atom`)\n* `url_path` (`:string`)\n* `disabled` (`:boolean`) - Defaults to `false`.\n* `color` (`:string`) - Defaults to `\"white\"`.","ref":"LiveUI.Components.Core.html#action_button/1-attributes"},{"type":"function","title":"Slots - LiveUI.Components.Core.action_button/1","doc":"* `inner_block` (required)","ref":"LiveUI.Components.Core.html#action_button/1-slots"},{"type":"function","title":"LiveUI.Components.Core.action_modal/1","doc":"Renders component for an action.","ref":"LiveUI.Components.Core.html#action_modal/1"},{"type":"function","title":"Attributes - LiveUI.Components.Core.action_modal/1","doc":"* `live_action` (`:atom`) (required)\n* `action` (`:atom`) (required)\n* `return_to` (`:string`) (required)","ref":"LiveUI.Components.Core.html#action_modal/1-attributes"},{"type":"function","title":"Slots - LiveUI.Components.Core.action_modal/1","doc":"* `inner_block` (required)\n* `header`\n* `subheader`","ref":"LiveUI.Components.Core.html#action_modal/1-slots"},{"type":"function","title":"LiveUI.Components.Core.alert/1","doc":"","ref":"LiveUI.Components.Core.html#alert/1"},{"type":"function","title":"LiveUI.Components.Core.badge/1","doc":"","ref":"LiveUI.Components.Core.html#badge/1"},{"type":"function","title":"LiveUI.Components.Core.button/1","doc":"","ref":"LiveUI.Components.Core.html#button/1"},{"type":"function","title":"LiveUI.Components.Core.card/1","doc":"","ref":"LiveUI.Components.Core.html#card/1"},{"type":"function","title":"LiveUI.Components.Core.card_content/1","doc":"","ref":"LiveUI.Components.Core.html#card_content/1"},{"type":"function","title":"LiveUI.Components.Core.color_scheme_switch/1","doc":"A button that switches between light and dark modes.\n\nPairs with css-theme-switch.js\n\nTaken from https://github.com/petalframework/petal_boilerplate/blob/21a70bec1060872d1ee88a23d31f8d2fcfd4cfc0/lib/petal_boilerplate_web/components/core_components.ex#L587\n\n    <.color_scheme_switch_js />","ref":"LiveUI.Components.Core.html#color_scheme_switch/1"},{"type":"function","title":"LiveUI.Components.Core.color_scheme_switch_js/1","doc":"Used for switching dark/light color schemes.\n\nThis needs to be inlined in the   because it will set a class on the document,\nwhich affects all \"dark\" prefixed classed (eg. dark:text-white).\nIf you do this in the body or a separate javascript file then when in dark mode,\nthe page will flash in light mode first before switching to dark mode.\n\nUtilized by `color-scheme-hook.js`.\n\nTaken from\nhttps://github.com/petalframework/petal_boilerplate/blob/21a70bec1060872d1ee88a23d31f8d2fcfd4cfc0/lib/petal_boilerplate_web/components/core_components.ex#L630\n\n    <.color_scheme_switch_js />","ref":"LiveUI.Components.Core.html#color_scheme_switch_js/1"},{"type":"function","title":"LiveUI.Components.Core.field/1","doc":"","ref":"LiveUI.Components.Core.html#field/1"},{"type":"function","title":"LiveUI.Components.Core.field_error/1","doc":"","ref":"LiveUI.Components.Core.html#field_error/1"},{"type":"function","title":"LiveUI.Components.Core.field_label/1","doc":"","ref":"LiveUI.Components.Core.html#field_label/1"},{"type":"function","title":"LiveUI.Components.Core.h1/1","doc":"","ref":"LiveUI.Components.Core.html#h1/1"},{"type":"function","title":"LiveUI.Components.Core.h2/1","doc":"","ref":"LiveUI.Components.Core.html#h2/1"},{"type":"function","title":"LiveUI.Components.Core.h3/1","doc":"","ref":"LiveUI.Components.Core.html#h3/1"},{"type":"function","title":"LiveUI.Components.Core.h4/1","doc":"","ref":"LiveUI.Components.Core.html#h4/1"},{"type":"function","title":"LiveUI.Components.Core.h5/1","doc":"","ref":"LiveUI.Components.Core.html#h5/1"},{"type":"function","title":"LiveUI.Components.Core.hide/2","doc":"Taken from phoenix core.","ref":"LiveUI.Components.Core.html#hide/2"},{"type":"function","title":"LiveUI.Components.Core.hide_modal/2","doc":"Taken from phoenix core.","ref":"LiveUI.Components.Core.html#hide_modal/2"},{"type":"function","title":"LiveUI.Components.Core.icon/1","doc":"Taken from phoenix core.","ref":"LiveUI.Components.Core.html#icon/1"},{"type":"function","title":"Attributes - LiveUI.Components.Core.icon/1","doc":"* `name` (`:string`) (required)\n* `class` (`:string`) - Defaults to `nil`.","ref":"LiveUI.Components.Core.html#icon/1-attributes"},{"type":"function","title":"LiveUI.Components.Core.icon_button/1","doc":"","ref":"LiveUI.Components.Core.html#icon_button/1"},{"type":"function","title":"LiveUI.Components.Core.modal/1","doc":"Taken from phoenix core.","ref":"LiveUI.Components.Core.html#modal/1"},{"type":"function","title":"Attributes - LiveUI.Components.Core.modal/1","doc":"* `id` (`:string`) (required)\n* `show` (`:boolean`) - Defaults to `false`.\n* `on_cancel` (`Phoenix.LiveView.JS`) - Defaults to `%Phoenix.LiveView.JS{ops: []}`.\n* `class` (`:string`) - Defaults to `nil`.","ref":"LiveUI.Components.Core.html#modal/1-attributes"},{"type":"function","title":"Slots - LiveUI.Components.Core.modal/1","doc":"* `inner_block` (required)","ref":"LiveUI.Components.Core.html#modal/1-slots"},{"type":"function","title":"LiveUI.Components.Core.p/1","doc":"","ref":"LiveUI.Components.Core.html#p/1"},{"type":"function","title":"LiveUI.Components.Core.pagination/1","doc":"","ref":"LiveUI.Components.Core.html#pagination/1"},{"type":"function","title":"LiveUI.Components.Core.progress/1","doc":"","ref":"LiveUI.Components.Core.html#progress/1"},{"type":"function","title":"LiveUI.Components.Core.relation_link/1","doc":"Renders link to parent record.","ref":"LiveUI.Components.Core.html#relation_link/1"},{"type":"function","title":"Attributes - LiveUI.Components.Core.relation_link/1","doc":"* `record` (`:map`)","ref":"LiveUI.Components.Core.html#relation_link/1-attributes"},{"type":"function","title":"LiveUI.Components.Core.show/2","doc":"Taken from phoenix core.","ref":"LiveUI.Components.Core.html#show/2"},{"type":"function","title":"LiveUI.Components.Core.show_modal/2","doc":"Taken from phoenix core.","ref":"LiveUI.Components.Core.html#show_modal/2"},{"type":"function","title":"LiveUI.Components.Core.ul/1","doc":"","ref":"LiveUI.Components.Core.html#ul/1"},{"type":"module","title":"LiveUI.Formatters","doc":"Functions for controlling how the fields are displayed on the screen.\n\n    import LiveUI.Formatters\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(\n        email: {&mask/2, [left: 4]},\n        bio: &markdown/1,\n        website: {&link_/1, %{name: \"Web\"}},\n        role: &String.capitalize/1\n      )\n    end\n\n\nAdding custom formatter is documented in `LiveUI.Protocol.Utils.add_formatters/2`.","ref":"LiveUI.Formatters.html"},{"type":"function","title":"LiveUI.Formatters.copy/1","doc":"Renders a copy icon next to the value.\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(email: &LiveUI.Formatters.copy/1,\n    end","ref":"LiveUI.Formatters.html#copy/1"},{"type":"function","title":"Attributes - LiveUI.Formatters.copy/1","doc":"* `value` (`:string`) - field value that will be copied into clipboard.\n* `field` (`:string`) - field name used as a part of DOM id - `copy- `.\n* `class` (`:string`) - CSS class. Defaults to `nil`.","ref":"LiveUI.Formatters.html#copy/1-attributes"},{"type":"function","title":"LiveUI.Formatters.link_/1","doc":"Renders a link with an icon.\n\n    def index_view(user) do\n      super(user)\n      |> website: &LiveUI.Formatters.link_/1\n      |> website: {&LiveUI.Formatters.link_/1, %{name: \"Website\"}}\n    end","ref":"LiveUI.Formatters.html#link_/1"},{"type":"function","title":"Attributes - LiveUI.Formatters.link_/1","doc":"* `name` (`:string`) - Defaults to `\"\"`.","ref":"LiveUI.Formatters.html#link_/1-attributes"},{"type":"function","title":"LiveUI.Formatters.markdown/1","doc":"Renders markdown as HTML.\n\n    def index_view(user) do\n      super(user)\n      |> bio: &LiveUI.Formatters.markdown/1\n    end","ref":"LiveUI.Formatters.html#markdown/1"},{"type":"function","title":"LiveUI.Formatters.mask/2","doc":"Masks a value with asterisks.\n\n      def index_view(user) do\n        super(user)\n        |> add_formatters(email: {&LiveUI.Formatters.mask/2, [right: 4]})\n      end","ref":"LiveUI.Formatters.html#mask/2"},{"type":"module","title":"LiveUI.Protocol.Utils","doc":"Utility functions for modifying default `LiveUI` implementation.","ref":"LiveUI.Protocol.Utils.html"},{"type":"function","title":"LiveUI.Protocol.Utils.add_action/5","doc":"Adds custom action to `Index` or `Show` view that points to user defined `Phoenix.LiveComponent` module.\n\nCustom component will be accessible with an action button and its content will be rendered inside the modal.\nThe action's URL is generated as `/ / / / ` and is set automatically by `LiveUI.Router.live_ui/3`.\nThe component for `Show` view has access to `@record` assign that holds database record.\n\nAdd custom `:api_key` action to `Index` view:\n\n    defimpl LiveUI, for: MyApp.Member.Contact do\n      use LiveUI.Protocol\n\n      def index_view(record) do\n        super(record)\n        |> add_action(:api_key, \"Get api key\", MyAppWeb.Member.ContactLive.ApiKey)\n      end\n    end\n\nCustom component that generates the api key could use `@current_user` from the socket:\n\n    defmodule MyAppWeb.Member.ContactLive.ApiKey do\n      use Phoenix.LiveComponent\n      import LiveUI.Components.Core\n\n      def update(assigns, socket) do\n        {:ok, socket |> assign(assigns) |> assign(:api_key, false)}\n      end\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <.p>Use this key to access our API</.p>\n          <.h3>Get api key</.h3>\n\n          <.p :if={@api_key}>\n             \n          </.p>\n\n          <.p :if={!@api_key}>\n            <.button variant=\"outline\" phx-click=\"get_api_key\" phx-target={@myself}>\n              Generate new key\n            </.button>\n          </.p>\n\n          <.p class=\"my-4\">\n            Key will expire in 180 days. Notification will be sent to <%= @current_user.email %>.\n          </.p>\n          <.link patch={@index_path}>Back</.link>\n         \n        \"\"\"\n      end\n\n      def handle_event(\"get_api_key\", _params, socket) do\n        {\n          :noreply,\n          socket |> assign(api_key: MyApp.Member.User.get_api_key(socket.assigns.current_user))\n        }\n      end\n    end\n\n`LiveUI.Formatters.copy/1` is built-in component that shows an icon next to the value to copy it to the clipboard.\n`p` and `h3` components are from `LiveUI.Components.Core` which delegates the calls to `PetalComponents`.\n\nIt is also possible to add extra custom values to the socket assigns of the parent view by overriding mount callback,\ncurrent_user will be automatically available if set in `Phoenix.LiveView.Router.live_session/3`.\n\nBy default `:allowed` field is set to `true`; it also could point to a function that controls who has the access to the component.","ref":"LiveUI.Protocol.Utils.html#add_action/5"},{"type":"function","title":"LiveUI.Protocol.Utils.add_batch_action/5","doc":"Adds custom batch action to `Index` view that points to user defined `Phoenix.LiveComponent` module.\n\nCustom component will be accessible with an action button and its content will be rendered inside the modal.\nThe action's URL is generated as `/ / / ` and is set automatically by `LiveUI.Router.live_ui/3`.\nThe component has access to `@selected` assign that holds ids of selected records.\n\nAdd custom `:deactivate` batch action:\n\n    defimpl LiveUI, for: MyApp.Admin.User do\n      def index_view(user) do\n        super(user)\n        |> add_batch_action(:deactivate, \"Deactivate\", MyAppWeb.Admin.UserLive.Deactivate)\n      end\n    end\n\nCustom component that will deactivate users:\n\n    defmodule LiveUIWeb.Admin.UserLive.Deactivate do\n      @moduledoc false\n\n      use Phoenix.LiveComponent\n      alias Phoenix.LiveView.JS\n      import LiveUI.Components.Core\n      import Ecto.Query\n\n      def update(assigns, socket) do\n        users =\n          from(users in LiveUI.Admin.User, where: users.id in ^assigns.selected)\n          |> LiveUI.Config.repo().all\n\n        {:ok, socket |> assign(assigns) |> assign(:users, users)}\n      end\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <.p>Deactivating multiple users</.p>\n          <.h3>\n            Are you sure you want to deactivate <%= length(@selected) %> user(s)?\n          </.h3>\n\n          <.ul class=\"my-6\">\n             \n              <%= user.name %> (<%= user.email %>)\n             \n          </.ul>\n\n          <.button link_type=\"live_patch\" to={@index_path} variant=\"outline\">\n            Back\n          </.button>\n\n          <.link\n            class=\"px-2 text-red-600\"\n            tabindex=\"-1\"\n            phx-target={@myself}\n            phx-click={JS.push(\"deactivate\", value: %{selected: @selected})}\n          >\n            Deactivate\n          </.link>\n         \n        \"\"\"\n      end\n\n      def handle_event(\"deactivate\", %{\"selected\" => selected}, socket) do\n        from(users in LiveUI.Admin.User, where: users.id in ^selected)\n        |> LiveUI.Config.repo().update_all(set: [active: false])\n\n        {:noreply,\n        socket\n        |> put_flash(:info, \"#{length(selected)} user(s) are deactivated successfully.\")\n        |> push_navigate(to: socket.assigns.index_path)}\n      end\n    end","ref":"LiveUI.Protocol.Utils.html#add_batch_action/5"},{"type":"function","title":"LiveUI.Protocol.Utils.add_formatters/2","doc":"Format fields with CSS classes, functions or components.\n\nMultiple formatters for a field are set as a list of formatters.\n\n    import LiveUI.Formatters\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(\n        email: {&mask/2, [left: 4]},\n        bio: &markdown/1,\n        website: {&link_/1, %{name: \"Web\"}},\n        role: &String.upcase/1\n      )\n    end\n\nNOTE: Formatting enum field will also format its values in dropdown input in related form.\nThis could be improved in future versions with support for custom input components.\n\n#","ref":"LiveUI.Protocol.Utils.html#add_formatters/2"},{"type":"function","title":"Formatting with CSS class - LiveUI.Protocol.Utils.add_formatters/2","doc":"When formatter is a string it will wrap the value with a span element with CSS class.\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(email: \"text-green-700\")\n    end\n\n#","ref":"LiveUI.Protocol.Utils.html#add_formatters/2-formatting-with-css-class"},{"type":"function","title":"Formatting with a function - LiveUI.Protocol.Utils.add_formatters/2","doc":"Field value is passed as an argument to a function with arity of 1.\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(role: &String.upcase/1)\n    end\n\n Record and field values are passed to a function with arity of 2.\n\n    # show prices as $1,234.00\n    def index_view(product) do\n      super(product)\n      |> ignore_fields([:currency])\n      |> add_formatters(price: &MyAppWeb.Formatters.money/2)\n    end\n\n    def money(record, value) do\n      Money.new(Map.get(record, :currency), value)\n    end\n\nExtra options are set as keyword list in 2-tuple.\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(email: {&mask/2, [left: 4]})\n    end\n\n#","ref":"LiveUI.Protocol.Utils.html#add_formatters/2-formatting-with-a-function"},{"type":"function","title":"Formatting with a component - LiveUI.Protocol.Utils.add_formatters/2","doc":"If the function has a map as an argument it is treated as a component.\nComponent assigns are passed as `%{field: field, value: value, record: record}`.\n\n    # built-in component\n    def index_view(user) do\n      super(user)\n      |> add_formatters(email: &LiveUI.Formatters.copy/1\n    end\n\n    # custom component\n    def index_view(user) do\n      super(user)\n      |> add_formatters(email: &MyAppWeb.Components.email/1\n    end\n\nExtra options are set as map in 2-tuple.\n\n    def index_view(user) do\n      super(user)\n      |> add_formatters(website: {&link/1, %{name: \"Web\"}})\n    end","ref":"LiveUI.Protocol.Utils.html#add_formatters/2-formatting-with-a-component"},{"type":"function","title":"LiveUI.Protocol.Utils.configure_inputs/3","doc":"Change field's default input type.\n\n    # change input from text to textarea\n    def index_view(company) do\n      super(company)\n        |> configure_inputs([:new], description: \"textarea\")\n    end\n\nNOTE: - `textarea` is currently used for `map` and `array` ecto types until we add configurable components for inputs.","ref":"LiveUI.Protocol.Utils.html#configure_inputs/3"},{"type":"function","title":"LiveUI.Protocol.Utils.ignore_fields/2","doc":"Don't show fields in `Index` or `Show` view.\n\n    def index_view(user) do\n      super(user)\n      |> ignore_fields([:updated_at, :inserted_at])\n    end","ref":"LiveUI.Protocol.Utils.html#ignore_fields/2"},{"type":"function","title":"LiveUI.Protocol.Utils.ignore_fields/3","doc":"Don't show fields in actions.\n\n    # remove :confirmed_at from create form\n    def index_view(user) do\n      super(user)\n      |> ignore_fields(:new, [:confirmed_at])\n    end\n\n    # remove :email and :company_id from edit form\n    def show_view(user) do\n      super(user)\n      |> ignore_fields([:edit], [:email, :company_id])\n    end\n\nNOTE: fields can be disabled globally in `config.exs`:\n\n    config :live_ui,\n      ignored_fields: [:token, :hashed_password, :first_version_id, :current_version_id]","ref":"LiveUI.Protocol.Utils.html#ignore_fields/3"},{"type":"function","title":"LiveUI.Protocol.Utils.set_optional_fields/3","doc":"All fields are required in `Ecto.Changeset` unless they are marked as optional.\n\n    def index_view(user) do\n      super(user)\n        |> set_optional_fields(:new, [:age])\n    end","ref":"LiveUI.Protocol.Utils.html#set_optional_fields/3"},{"type":"module","title":"LiveUI.Router","doc":"Macro which defines a set of routes for given `Ecto.Schema` module.","ref":"LiveUI.Router.html"},{"type":"macro","title":"LiveUI.Router.live_ui/3","doc":"Helper that generates all required routes for a resource.\n\n    # router.ex\n    scope \"/\", MyAppWeb do\n      scope \"/admin\", Admin do\n\n        # generate all routes\n        live_ui(\"/companies\", UserLive, MyApp.Admin.User)\n\n        # or define each one\n        live(\"/users\", UserLive.Index, :index)\n        live(\"/users/new\", UserLive.Index, :new)\n        live(\"/users/:id\", UserLive.Show, :show)\n        live(\"/users/:id/edit\", UserLive.Show, :edit)\n        live(\"/users/:id/delete\", UserLive.Show, :delete)\n      end\n    end\n\n`UserLive.Index` and `UserLive.Show` modules should call the macro to get basic `Phoenix.LiveView` implementation:\n\n    # lib/my_app_web/live/admin/user_live/index.ex\n    defmodule MyAppWeb.Admin.UserLive.Index do\n      use LiveUI.Views.Index, for: MyApp.Admin.User\n    end\n\n    # lib/my_app_web/live/admin/user_live/show.ex\n    defmodule MyAppWeb.Admin.UserLive.Index do\n      use LiveUI.Views.Show, for: MyApp.Admin.User\n    end\n\nThese modules will provide code to display and process data which is customized\nvia `LiveUI` protocol.\n\n`live_ui/3` will also generate routes for any custom action defined via `LiveUI` protocol:\n\n    live(\"/users/custom\", UserLive.Index, :custom)\n    live(\"/users/:id/custom\", UserLive.Show, :custom)","ref":"LiveUI.Router.html#live_ui/3"},{"type":"extras","title":"LiveUI","doc":"# LiveUI\n\nSet of macros for building `Phoenix.LiveView` modules to help manage database records.\n\nThis is done by configuring default implementation of `LiveUI` for a given `Ecto.Schema` struct and\ninitializing basic Index and Show live modules. This will in turn build the infrastructure for displaying\nand processing data without need to write boilerplate code associated with `m:Phoenix.LiveView#callbacks` callbacks and HEEX templates.\n\n`LiveUI` has the following features:\n\n* autogenerated views for CRUD operations\n* basic protocol implementation with sensible defaults\n* sortable tables with pagination\n* advanced search form\n* format values with css, functions and components\n* handles ownership to current user\n* handles relations\n* handles file uploads\n* support to override functions for data processing\n* support to override live view callbacks\n* custom actions for operations on single and multiple records\n* audit with record versioning\n* dark and light mode\n* debug mode that displays socket assigns\n\n`LiveUI` is possible because of these libraries:\n\n* `Cldr`\n* `Ecto`\n* `Faker`\n* `Flop`\n* `LiveInspect`\n* `LiveSelect`\n* `PaperTrail`\n* `PetalComponents`\n* `Phoenix`","ref":"readme.html"},{"type":"extras","title":"Installation - LiveUI","doc":"* add entry to `content` list in `assets/tailwind.config.js` - `../lib/live_ui/**/*.*ex`\n* add line to `assets/css/app.css` - `@import '../../../live_ui/lib/live_ui/assets/default.css'`\n* modify `root.html.heex` template - ` `\n* add `js-cookie` to `js/vendor` for theme switcher -  \n* add theme switcher hook to `assets/js/app.js`\n\n  ```javascript\n  const ColorSchemeHook = {\n    deadViewCompatible: true,\n    mounted() {\n      this.init()\n    },\n    updated() {\n      this.init()\n    },\n    init() {\n      initScheme()\n      this.el.addEventListener(\"click\", window.toggleScheme)\n    },\n  }\n  ```\n\n* add switcher code to head in `root.html.heex` - ` `\n* add switcher icon to body in `root.html.heex` - ` `\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Cldr - LiveUI","doc":"Add `Cldr` module:\n\n``` elixir\n  defmodule MyApp.Cldr do\n    use Cldr,\n      default_locale: \"en\",\n      otp_app: :my_app,\n      providers: [Cldr.Number, Money, Cldr.Calendar, Cldr.DateTime],\n      gettext: MyAppWeb.Gettext\n  end\n```\n\n#","ref":"readme.html#cldr"},{"type":"extras","title":"LiveSelect - LiveUI","doc":"Add hook to `assets/js/app.js` -  \n\n#","ref":"readme.html#liveselect"},{"type":"extras","title":"Petal components - LiveUI","doc":"Instalation guide -","ref":"readme.html#petal-components"},{"type":"extras","title":"Configuration - LiveUI","doc":"```elixir\n# live ui\nconfig :live_ui,\n  debug: true,\n  repo: MyApp.Repo,\n  cldr: MyApp.Cldr,\n  ignored_fields: [:token, :hashed_password, :first_version_id, :current_version_id]\n\n# petal components\nconfig :petal_components, :error_translator_function, {MyAppWeb.CoreComponents, :translate_error}\n\n# flop\nconfig :flop, repo: MyApp.Repo, default_limit: 10, max_limit: 100\n\n# cldr\nconfig :ex_cldr, default_backend: LiveUI.Cldr\nconfig :my_app, LiveUI.Cldr, locales: [\"en\"]\n\n# paper trail (optional)\nconfig :paper_trail,\n  repo: MyApp.Repo,\n  originator: [name: :user, model: MyApp.User],\n  strict_mode: true\n```\n\nThere is a sample aplication bundled with the project that has all these set up, just run\n`mix ecto.setup` to generate some random data and start the server with `iex -S mix phx.server`.\nAdmin credentials are `admin@example.com/password`.","ref":"readme.html#configuration"},{"type":"extras","title":"Minimal setup - LiveUI","doc":"Ecto schema:\n\n```elixir\ndefmodule MyApp.User do\n  use Ecto.Schema\n\n  schema \"users\" do\n    field :name, :string\n    field :email, :string\n    timestamps()\n  end\nend\n```\n\nRouter entry:\n\n```elixir\nscope \"/\", MyAppWeb do\n  live_ui(\"/users\", UserLive, MyApp.User)\nend\n```\n\nProtocol file:\n\n```elixir\ndefimpl LiveUI, for: MyApp.User do\n  use LiveUI.Protocol\nend\n```\n\nLive modules:\n\n```elixir\ndefmodule MyAppWeb.UserLive.Index do\n  use LiveUI.Views.Index, for: MyApp.User\nend\n\ndefmodule MyAppWeb.UserLive.Show do\n  use LiveUI.Views.Show, for: MyApp.User\nend\n```","ref":"readme.html#minimal-setup"},{"type":"extras","title":"Is it any good? - LiveUI","doc":"[Yes](https://news.ycombinator.com/item?id=3067434)","ref":"readme.html#is-it-any-good"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.1.0 (2024-03-03) - Changelog","doc":"First version","ref":"changelog.html#0-1-0-2024-03-03"}],"content_type":"text/markdown"}